<?php
namespace Teknisa\MAN\Controllers;


use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\MAN\Utilities\ChunkReadFilter;
use Teknisa\MAN\Services;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\Controller\BirtReport;
use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DTO\Response\Error;   
use Teknisa\Libs\Util\XLS\FileLoader;
use Teknisa\Libs\Util\XLS\FileReader;

class Consumidor extends  \Teknisa\MAN\Controllers\TeknisaCrud {

    protected $salesMessage;
    protected $consumidorService;
	protected $util;
	protected $SMTPService;
    protected $saasParameters;
    protected $formatBirt;
    public static $logMessages = array();
    
    const dataSourceTemplate = 'template_cadConsumidor';

    public function __construct($crudService, $entityManager, $parameterBag, Manager $dataSourceManager, $salesMessage, $consumidorService, $util, $SMTPService, 
                                   $env,  $birtReport,  $reportService) {
        $this->crudService       = $crudService;
        $this->salesMessage      = $salesMessage;
        $this->consumidorService = $consumidorService;
		$this->util              = $util;
		$this->SMTPService       = $SMTPService;
        // $this->saasParameters    = $saasParameters;
        $this->dataSourceManager = $dataSourceManager;
        $this->parameterBag      = $parameterBag;
        $this->entityManager     = $entityManager;
        $this->env               = $env;
        $this->birtReport        = $birtReport;
        $this->reportService     = $reportService;
        
        
        
    }
    
    public static function escreverLog ($data) {
        array_push(self::$logMessages, $data);
    }
    
    public static function escreverLogArray($array) {
        foreach ($array as $linha) {
            self::escreverLog($linha);
        }
    }
    
    public static function getLog () {
        return self::$logMessages;
    }

    private function generateLink ($baseUrl, $hash) {
		return $baseUrl.'?'.$hash;
	}

    private function buildHashCode($CDCONSUMIDOR) {
		$time = microtime(true);
		

		$hash = $this->util->encrypt($CDCONSUMIDOR.$time.rand(0, 99999999));
		$hash2 = $this->util->encrypt(rand(0,335689));
		$hash = $hash.$hash2;
		$hashArray = str_split($hash);
		$operatorArray = str_split($CDCONSUMIDOR);

		$parImpar = rand(0, 1);

		$finallyHash = "";
		foreach ($hashArray as $key => $hashValue) {
			if ($key % 2 == $parImpar && count($operatorArray) > 0) {
				$finallyHash .= array_shift($operatorArray);
			} else {
				$finallyHash .= $hashValue;
			}
		}
		$finallyHash .= $parImpar;
		return $finallyHash;
	}

	public function forgotPassword(Request\Filter $request, Response $response){
		try {
			$params = $request->getFilterCriteria()->getConditions();
			$NMLOGINCONS = $params[0]['value'];
            $DSEMAILCONS = $params[1]['value'];
			$baseUrl     = $params[2]['value'];

			//transforma em minusculo todos os caracters do e-mail digitados
			$NMLOGINCONS = strtolower($NMLOGINCONS);
			$activatedCards = $this->consumidorService->checkActivateCardByNMLOGINCONS($NMLOGINCONS);
			$consumerData = $this->consumidorService->getConsumerByLogin($NMLOGINCONS);
			if (empty($consumerData)) {
				$response->addMessage(new Message($this->salesMessage->getMessage(97)));
			} else {
				if (empty($activatedCards)) {  //testa se existem cartoes ativados
					$response->addMessage(new Message($this->salesMessage->getMessage(99)));
				} else {
					$CDCONSUMIDOR = $consumerData[0]['CDCONSUMIDOR'];
					$CDCLIENTE = $consumerData[0]['CDCLIENTE'];
					$hashConsumidor = $this->buildHashCode($CDCONSUMIDOR);
					$hashCliente = $this->buildHashCode($CDCLIENTE);
					$hash = $hashConsumidor.$hashCliente;
					$linkForChange = $this->generateLink($baseUrl, $hash);
					$NMCONSUMIDOR  = $consumerData[0]['NMCONSUMIDOR'];

					$subject  = $this->salesMessage->getMessage('101');
					$sender = $this->salesMessage->getMessage('103');
					$content =
						'<p>Ola '. $NMCONSUMIDOR. '!</p>'.
						'<p>Acesse o link a seguir para alterar sua senha: <b>'.
						'<a href='.$linkForChange.'>'.'Clique Aqui'.'</a>'.'</b> </p>'
					;

                    //Check if consumer has an email, if not check if NMCONSUMIDOR is a valid email
                    if ($DSEMAILCONS != null) {
					   $this->SMTPService->sendEmail($NMCONSUMIDOR, $DSEMAILCONS, $sender, $subject, $content);
					} else if(is_string(filter_var($NMLOGINCONS, FILTER_VALIDATE_EMAIL))) {
                        $this->SMTPService->sendEmail($NMCONSUMIDOR, $NMLOGINCONS, $sender, $subject, $content);
                    } else {
                        $response->addMessage(new Message($this->salesMessage->getMessage(64)));
                    }

                    $this->consumidorService->updateHashAlteraSenhaConsu($CDCONSUMIDOR, $CDCLIENTE, $hash);

					$response->addMessage(new Message($this->salesMessage->getMessage(96)));
				}
			}
		} catch (\Exception $e) {
			$response->addMessage(new Message($e->getMessage()));
		}
	}

    public function buscaConsumidores(Request\Filter $request, Response $response) {
        $params = $request->getFilterCriteria()->getConditions();
        
        $NRORG = $params[0]['value'];
        
        $CDCLIENTE = $params[1]['value'];
        $buscaConsumidores = $this->consumidorService->buscaConsumidores($CDCLIENTE, $NRORG);
        
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ConsumidoresRepo', $buscaConsumidores));
        
    }

    public function buscaConsClienteBeneficio(Request\Filter $request, Response $response){
        $params = $request->getFilterCriteria()->getConditions();
    
        $CDCLIENTE  = $params[0]['value'];
        $CDCAMPANHA = $params[1]['value'];

         $buscaConsumidores = $this->consumidorService->buscaConsClienteBeneficio($CDCLIENTE, $CDCAMPANHA,1);
         $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BuscaConsClienteBenRepo', $buscaConsumidores));
    }

    public function buscaConsumidor(Request\Filter $request, Response $response){
        $params = $request->getFilterCriteria()->getConditions();
        $CDIDCONSUMID = "CDE00000000".$params[0]['value'];
        $NRORG = $params[1]['value'];
        
        $buscaConsumidor = $this->consumidorService->buscaConsumidor($CDIDCONSUMID,$NRORG);
        if(empty($buscaConsumidor)){
            $response->addMessage(new Message($this->salesMessage->getMessage(28)));
        }else{
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BuscaConsumidorRepo', $buscaConsumidor));
        }
    }

    public function buscaConsumidorCompleto(Request\Filter $request, Response $response){
        $params = $request->getFilterCriteria()->getConditions();

        $CDCLIENTE = $params[0]['value'];
        $CDCONSUMIDOR = $params[1]['value'];
        $NRORG = $params[2]['value'];
        
        $buscaConsumidorCompleto = $this->consumidorService->buscaConsumidorCompleto($CDCLIENTE, $CDCONSUMIDOR, $NRORG);

        if(empty($buscaConsumidorCompleto)){
            $response->addMessage(new Message($this->salesMessage->getMessage(28)));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BuscaConsumidorCompletoRepo', $buscaConsumidorCompleto));
        }else{
	        switch ($buscaConsumidorCompleto[0]['IDTPVENDACONS']) {
	        	case '1':
	           		$buscaConsumidorCompleto[0]['NMTPVENDACONS'] = "1 - Venda Débito Consumidor";
	           	break;
	           	case '2':
	           		$buscaConsumidorCompleto[0]['NMTPVENDACONS'] = "2 - Venda Crédito Pessoal";
	           	break;
	           	case '3':
	           		$buscaConsumidorCompleto[0]['NMTPVENDACONS'] = "3 - Venda a Vista";
	           	break;
	           	case '4':
	           		$buscaConsumidorCompleto[0]['NMTPVENDACONS'] = "4 - Venda Débito Consumidor/Crédito Pessoal";
	           	break;
	           	case '5':
	           		$buscaConsumidorCompleto[0]['NMTPVENDACONS'] = "5 - Venda Débito Consumidor/a Vista";
	           	break;
	           	case '6':
	           		$buscaConsumidorCompleto[0]['NMTPVENDACONS'] = "6 - Venda Crédito Pessoal/a Vista";
	           	break;
	           	case '7':
	           		$buscaConsumidorCompleto[0]['NMTPVENDACONS'] = "7 - Todos";
	           	break;
	        }
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BuscaConsumidorCompletoRepo', $buscaConsumidorCompleto));
        }
    }
    
    public function findTipoConsumidor(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'TipoCons';
        $this->parameterBagColumns = array('NRORG', 'CDCLIENTE');
        parent::find($request,$response);
    }
    
     public function findTipoVendas(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'TipoVendaCons';
        $this->parameterBagColumns = array('NRORG');
        parent::find($request,$response);
    }
    
    public function getTipoConsumidor(Request\Filter $request, Response $response){
       $params = $request->getFilterCriteria()->getConditions();
       if(!empty($params) && count($params) === 2 ) {
           $tipoConsumidor = $this->consumidorService->buscaTipoConsumidorPorCliente($params[0]['value'], $params[1]['value']);
           
           if(!$tipoConsumidor) {
              $tipoConsumidor = $this->consumidorService->buscaTipoConsumidor();
           }
           $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('TipoConsRepo', $tipoConsumidor));
       }
   }

    public function resetSenha(Request\Filter $request, Response $response){
        $params = $request->getFilterCriteria()->getConditions();

		$CDCLIENTE = $params[0]['value'];
        $CDCONSUMIDOR = $params[1]['value'];

        $resetSenha = $this->consumidorService->resetSenha($CDCLIENTE, $CDCONSUMIDOR);

        $response->addMessage(new Message($this->salesMessage->getMessage(93)));
    }

    public function alteraConsumidor(Request\Filter $request, Response $response){
    	$params = $request->getFilterCriteria()->getConditions();
		$CDCLIENTE = $params[0]['value'];
        $CDCONSUMIDOR = $params[1]['value'];
 		$NMCONSUMIDOR = $params[2]['value'];
 		$IDSITCONSUMI = $params[3]['value'];
 		$CDPAIS = $params[4]['value'];
 		$SGESTADO = $params[5]['value'];
 		$CDMUNICIPIO = $params[6]['value'];
 		$CDTIPOCONS = $params[7]['value'];
 		$CDCCUSCLIE = $params[8]['value'];
 		$CDIDCONSUMID = $params[9]['value'];
 		$DSENDECONS = $params[10]['value'];
 		$NMBAIRCONS = $params[11]['value'];
 		$NRCEPCONS = $params[12]['value'];
 		$NRTELECONS = $params[13]['value'];
 		$NRRAMAL1CONS = $params[14]['value'];
 		$DSEMAILCONS = $params[15]['value'];
 		$IDVERSALDCON = $params[16]['value'];
 		$NRRGCONSUMID = $params[17]['value'];
 		$CDSENHACONS = $params[18]['value'];
 		$NRCPFRESPCON = $params[19]['value'];
 		$NMRESPCONS = $params[20]['value'];
 		$NMLOGINCONS = $params[21]['value'];
 		$IDSEXOCONS = $params[22]['value'];
 		$DTNASCCONS = $params[23]['value'];
 		$CDSENHASITE = $params[24]['value'];
 		$IDTPVENDACONS = $params[25]['value'];
 		$NRCELULARCONS = $params[26]['value'];
 		$CDBAIRRO = $params[27]['value'];
 		$IDIMPCPFCUPOM = $params[28]['value'];
 		$IDCADCONFLIBCON = $params[29]['value'];
 		$IDPERCONSPRODEX = $params[30]['value'];
 		$IDTPSELMANHA = $params[31]['value'];
 		$IDTPSEALMOCO = $params[32]['value'];
 		$IDTPSELTARDE = $params[33]['value'];
 		$CDSENHACONSUMIDORWEB = $params[34]['value'];
 		$DTVALIDADECONSUMIDOR = $params[35]['value'];
 		$VRLIMDEBCONS = $params[36]['value'];
 		$VRMAXDEBCONS = $params[37]['value'];
 		$VRAVIDEBCONS = $params[38]['value'];
 	    $IDATIVO = ($params[39]['value'] == NULL) ?  "S" : "N";
 	    $NRORG = $params[40]['value'];
        
		$novoConsumidor = $this->consumidorService->alteraConsumidor($CDCLIENTE, $CDCONSUMIDOR, $NMCONSUMIDOR, $IDSITCONSUMI, $CDPAIS, $SGESTADO, $CDMUNICIPIO, $CDTIPOCONS, $CDCCUSCLIE, $CDIDCONSUMID,
			   $DSENDECONS, $NMBAIRCONS, $NRCEPCONS, $NRTELECONS, $NRRAMAL1CONS, $DSEMAILCONS, $IDVERSALDCON, $NRRGCONSUMID, $CDSENHACONS, $NRCPFRESPCON,
			   $NMRESPCONS, $NMLOGINCONS, $IDSEXOCONS, $DTNASCCONS, $CDSENHASITE, $IDTPVENDACONS, $NRCELULARCONS, $CDBAIRRO, $IDIMPCPFCUPOM,
			   $IDCADCONFLIBCON, $IDPERCONSPRODEX, $IDTPSELMANHA, $IDTPSEALMOCO, $IDTPSELTARDE, $CDSENHACONSUMIDORWEB, $DTVALIDADECONSUMIDOR,
			   $VRLIMDEBCONS,$VRMAXDEBCONS,$VRAVIDEBCONS, $IDATIVO, $NRORG);
		
        if($IDATIVO != "N"){
            $response->addNotification(new Notification("Consumidor alterado com sucesso!", Notification::TYPE_SUCCESS));
        }
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('AlteraConsumidorRepo', $params));
    }

    public function novoConsumidor(Request\Filter $request, Response $response){
    	$params = $request->getFilterCriteria()->getConditions();
    	
		$CDCLIENTE = $params[0]['value'];
        $CDCONSUMIDOR = $params[1]['value'];
 		$NMCONSUMIDOR = $params[2]['value'];
 		$IDSITCONSUMI = $params[3]['value'];
 		$CDPAIS = $params[4]['value'];
 		$SGESTADO = $params[5]['value'];
 		$CDMUNICIPIO = $params[6]['value'];
 		$CDTIPOCONS = $params[7]['value'];
 		$CDCCUSCLIE = $params[8]['value'];
 		$CDIDCONSUMID = $params[9]['value'];
 		$DSENDECONS = $params[10]['value'];
 		$NMBAIRCONS = $params[11]['value'];
 		$NRCEPCONS = $params[12]['value'];
 		$NRTELECONS = $params[13]['value'];
 		$NRRAMAL1CONS = $params[14]['value'];
 		$DSEMAILCONS = $params[15]['value'];
 		$IDVERSALDCON = $params[16]['value'];
 		$NRRGCONSUMID = $params[17]['value'];
 		$CDSENHACONS = $params[18]['value'];
 		$NRCPFRESPCON = $params[19]['value'];
 		$NMRESPCONS = $params[20]['value'];
 		$NMLOGINCONS = $params[21]['value'];
 		$IDSEXOCONS = $params[22]['value'];
 		$DTNASCCONS = $params[23]['value'];
 		$CDSENHASITE = $params[24]['value'];
 		$IDTPVENDACONS = $params[25]['value'];
 		$NRCELULARCONS = $params[26]['value'];
 		$CDBAIRRO = $params[27]['value'];
 		$IDIMPCPFCUPOM = $params[28]['value'];
 		$IDCADCONFLIBCON = $params[29]['value'];
 		$IDPERCONSPRODEX = $params[30]['value'];
 		$IDTPSELMANHA = $params[31]['value'];
 		$IDTPSEALMOCO = $params[32]['value'];
 		$IDTPSELTARDE = $params[33]['value'];
 		$CDSENHACONSUMIDORWEB = $params[34]['value'];
 		$DTVALIDADECONSUMIDOR = $params[35]['value'];
 		$VRLIMDEBCONS = $params[36]['value'];
 		$VRMAXDEBCONS = $params[37]['value'];
 		$VRAVIDEBCONS = $params[38]['value'];
 		$NRORG = $params[39]['value'];
 		
        $checkCodigo = $this->consumidorService->checkConsumidores($CDCONSUMIDOR, $NRORG);
        
        
        if (count($checkCodigo) > 0){
            $response->addNotification(new Notification("Já existe um consumidor com esse Código!", Notification::TYPE_ALERT)); 
            $response->addDataSet(new DataSet('novoConsumidor', array(false)));
            
        }else{
    		$novoConsumidor = $this->consumidorService->novoConsumidor($CDCLIENTE, $CDCONSUMIDOR, $NMCONSUMIDOR, $IDSITCONSUMI, $CDPAIS, $SGESTADO, $CDMUNICIPIO, $CDTIPOCONS, $CDCCUSCLIE, $CDIDCONSUMID,
    			   $DSENDECONS, $NMBAIRCONS, $NRCEPCONS, $NRTELECONS, $NRRAMAL1CONS, $DSEMAILCONS, $IDVERSALDCON, $NRRGCONSUMID, $CDSENHACONS, $NRCPFRESPCON,
    			   $NMRESPCONS, $NMLOGINCONS, $IDSEXOCONS, $DTNASCCONS, $CDSENHASITE, $IDTPVENDACONS, $NRCELULARCONS, $CDBAIRRO, $IDIMPCPFCUPOM,
    			   $IDCADCONFLIBCON, $IDPERCONSPRODEX, $IDTPSELMANHA, $IDTPSEALMOCO, $IDTPSELTARDE, $CDSENHACONSUMIDORWEB, $DTVALIDADECONSUMIDOR,
    			   $VRLIMDEBCONS,$VRMAXDEBCONS,$VRAVIDEBCONS, $NRORG);
            $response->addDataSet(new DataSet('novoConsumidor', array(true)));
            
            $response->addNotification(new Notification("Consumidor salvo com sucesso!", Notification::TYPE_SUCCESS));
        }    
    }

    public function getCities(Request\Filter $request, Response $response){
        try {
            $params   = $request->getFilterCriteria()->getConditions();
            if(isset($params[0])){
                $SGESTADO = $params[0]['value'];
            }else{
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('CitiesRepository', array('nada' => 'nada' )));
                return 0;
            }
            if(isset($params[1])){
                $NMMUNICIPIO = $params[1]['value'];
            }else{
                $NMMUNICIPIO = "%%";
            }
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $FIRST = $request->getFilterCriteria()->getPageSize() * ($page-1);
            $LAST = ($FIRST + $pageSize);
            if($page > 1){
                $FIRST++;
            }
            $cities = $this->locationService->getCities($SGESTADO, $NMMUNICIPIO, $FIRST, $LAST);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('CitiesRepository', $cities));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    //SAAS DB connection through NRORG
    public function setInstitutionConnection(Request\Filter $request, Response $response){
        try {

            $params = $request->getFilterCriteria()->getConditions();
            $nrorg = $params[0]['value'];

            //$this->saasParameters->set('nrorg', $nrorg);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('SetConnectionRepo'));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function findConsumidorAltAutomatica (Request\Filter $request, Response $response){
        $params = $this->util->getParamsAsArray($request);
        $CDCLIENTE = implode('_', $params['CDCLIENTE']);
        $CDCLIENTECUSCLIE = implode('_', $params['CDCLIENTECUSCLIE']);
        $CDTIPOCONSCLIE = implode('_', $params['CODIGO']);
        $CDFILIAL = $params['CDFILIAL'];
        $NRORG = $params['NRORG'];
        
        $retorno = $this->consumidorService->findConsumidorAltAutomatica($CDCLIENTE, $CDCLIENTECUSCLIE, $CDTIPOCONSCLIE, $CDFILIAL, $NRORG);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ConsumidorAltAutomatica', $retorno));
        
    }
    public function findConsumidorRel (Request\Filter $request, Response $response){
        $params = $this->util->getParamsAsArray($request);
        
        $params['CDTIPOCONSCLIE'] = $params['CODIGO'];
        if($params['CDTIPOCONSCLIE'] == ''){
            unset($params['CDTIPOCONSCLIE']);
        };
        if($params['CDCLIENTE'] == ''){
            unset($params['CDCLIENTE']);
        };
        
        $params = $this->addParams($params,'CDCLIENTE');
        $params = $this->addParams($params,'CDTIPOCONSCLIE');
        
        $retorno = $this->consumidorService->findConsumidorRel($this->consumidorService::GET_CONSUMIDOR_REL,$params);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('consumidorRel', $retorno));
        
    }
    public function findTipoConsumidorRel (Request\Filter $request, Response $response){
        $params = $this->util->getParamsAsArray($request);
        if($params['CDCLIENTE'] == ''){
            unset($params['CDCLIENTE']);
        };
       
        $params      = $this->addParams($params,'CDCLIENTE');
        $params = $this->addParams($params,'CDTIPOCONSCLIE');
        $retorno = $this->consumidorService->findConsumidorRel($this->consumidorService::GET_TIPO_CONSUMIDOR_REL,$params);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('tipoConsumidorRel', $retorno));
        
    }
    
    private function addParams($conditions,$param){
        
        if(!array_key_exists($param, $conditions)) {
            $conditions[$param] = '';
            $conditions['T'.$param] = 'T';
        }else{
            $conditions[$param] = implode('_', $conditions[$param]);
            $conditions['T'.$param] = '';
        }
         
         return $conditions;
    }
    
    public function saveAltAutomatica(Request\DataSet $request, Response $response){
        try {
            $rows = $request->getDataSet()->getRows()[0];
            $applyeveryone= $rows['IDAPPLYEVERYONE'];
            if($applyeveryone=="S"){
                $retorno = $this->consumidorService->saveAltAutomaticaEveryone($rows);
            }else{
                $retorno = $this->consumidorService->saveAltAutomatica($rows);
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('consumidorAltAutomatica', array()));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function importarXLS(Request $request, Response $response){
        
        try {
            
            ini_set('max_execution_time', 1800);
            ini_set('memory_limit', '500M');
            
            $array = array(
                "path"         => '',
                "file"         => '',
                "status"       => '',
                "message"      => '',
                "controle"     => 1,
                "client"       => '',
                'totalSuccess' => 0,
                'totalError'   => 0,
                'nameFile'     => ''
            );
            
            $rows = $request->getRow();
            
            if(!empty($rows['log'])){
                self::escreverLogArray($rows['log']);
            }else{
                self::escreverLog("## Exportação Iniciada Data: ".date("d/m/Y H:i:s")." ##");
                self::escreverLog("## Arquivo exportado: ".$rows['nameFile']." ##");
            }
            
            if(!empty($rows['diretorio']) && !empty($rows['file'])){
                
                $array['path']         = $rows['diretorio'];
                $array['file']         = $rows['file'];
                $array['client']       = $rows['client'];
                $array['nameFile']     = $rows['nameFile'];
                $array['content']      = $rows['content'];
                $array['stop']         = false;
                $array['totalSuccess'] = $rows['tSuccess'];
                $array['totalError']   = $rows['tError'];
                
                $fileName = $rows['diretorio'].$rows['file'];
                
                $sheetsToLoad = array('Sheasdasdsadet1', 'Sheet1', 'Planilha1');
                $xlsFile = FileLoader::loadFromBase64($array['content'], $array['nameFile']);
                
                if(!$xlsFile) {
                    throw new \Exception("Falha ao abrir o arquivo");
                }
                
                $chunkSize    = 100;
                $startRow     = $rows['controle'];
                $tamanhoBloco = ($startRow + $chunkSize) - 1;
                
                $sheetData = $xlsFile->getActiveSheet()->toArray(null, true, true, true);
                
                //self::escreverLog("Loading WorkSheet for rows " . $startRow . " to " . ($startRow + $chunkSize - 1) . PHP_EOL );
                // $chunkFilter->setRows($startRow, $chunkSize);
                //self::escreverLog("Loading from " . $startRow . " to " . ($startRow + $chunkSize) );
            
                //$callStartTime = microtime(true);
                // $objPHPExcel = $xlsFile->load($fileName);
                //$callEndTime = microtime(true);
            
                //$callTime = $callEndTime - $callStartTime;
                //self::escreverLog('Call time to read Workbook was, ' .sprintf('%.4f',$callTime) . ", seconds" . PHP_EOL);
                //self::escreverLog(date('H:i:s') .', Current memory usage: ' . (memory_get_usage(true) / 1024 / 1024) . " MB" . PHP_EOL);
                
                if(!empty($sheetData) && count($sheetData) > 1) {
                    $retorno = $this->consumidorService->saveConsumidor($sheetData, $rows['client'], $startRow, $tamanhoBloco);
                    
                    if($retorno['status']){
                        $array['status']  = true;
                        $array['message'] = "Bloco de ".$startRow." a ".$tamanhoBloco." foram processados...";
                         
                        self::escreverLog("#######################################  Bloco de ".$startRow." a ".$tamanhoBloco." #######################################");
                    
                        if(!empty($retorno['logRowsSucess'])){
                            self::escreverLog("## Consumidores Inseridos com Sucesso ## ");
                            self::escreverLog($retorno['logRowsSucess']);
                            self::escreverLog("############################################################################");
                        }
                        
                        if(!empty($retorno['logRowsError'])){
                            self::escreverLog("## Consumidores que não foram Inseridos ## ");
                            self::escreverLog($retorno['logRowsError']);
                            self::escreverLog("############################################################################");
                        }
                        
                        if(!empty($retorno['logs'])){
                            self::escreverLog("## Erros Reportados na Exportação ## ");
                            self::escreverLog($retorno['logs']);
                            self::escreverLog("############################################################################");
                        }
                    }else{
                        $array['status'] = false;
                        $array['message'] = "Erro ao exportar!". (isset($retorno['message'])) ? $retorno['message'] : '';
                        self::escreverLog($array['message']);
                    }
                    
                    $array['controle']      = $startRow + $chunkSize;    
                    $array['totalSuccess'] += $retorno['rowsSucess'];    
                    $array['totalError']   += $retorno['rowsError'];    
                    
                    $array['log'] = $this->getLog();
                    
                    // verifica se todos os dados já foram inseridos
                    if(count($sheetData) <= $tamanhoBloco) {
                        self::escreverLog("Operação finalizada com sucesso! Data: ".date("d/m/Y H:i:s"));
                        self::escreverLog("Exportações finalizadas com sucesso: ".$array['totalSuccess']);
                        self::escreverLog("Exportações finalizadas com erros: ".$array['totalError']);
                        
                        $array['log']      = $this->getLog();
                        $array['stop']     = true;
                        $array['status']   = true;
                        $array['message']  = "Rotina finalizada com sucesso!";
                        $array['controle'] = 1;
                    }
                }else{
                    self::escreverLog("Operação finalizada com sucesso! Data: ".date("d/m/Y H:i:s"));
                    self::escreverLog("Exportações finalizadas com sucesso: ".$array['totalSuccess']);
                    self::escreverLog("Exportações finalizadas com erros: ".$array['totalError']);
                    
                    $array['log']      = $this->getLog();
                    $array['stop']     = true;
                    $array['status']   = true;
                    $array['message']  = "Rotina finalizada com sucesso!";
                    $array['controle'] = 1;
                }
                
                unset($sheetData, $xlsFile);
                
            }else{
                $array['status']  = false;
                $array['message'] = "Não há dados do arquivo para processar!";
            }
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('resultado', $array));
        } catch (\Exception $e) {
            
            self::escreverLog("## Erros Reportados na Exportação ## ");
            self::escreverLog($e->getMessage());
            
            
            $array = array(
                "status"  => false,
                "log"     => $this->getLog(),
                'message' => "Erro ao realizar rotina!"
            );
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('resultado', $array));
        }
    }
    
    public function tratarNomeArquivo($string){
       // pegando a extensao do arquivo
       $partes 	= explode(".", $string);
       $extensao 	= $partes[count($partes)-1];	
       // somente o nome do arquivo
       $nome	= preg_replace('/\.[^.]*$/', '', $string);	
       // removendo simbolos, acentos etc
       $a = 'ÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖØÙÚÛÜÝÞßàáâãäåæçèéêëìíîïðñòóôõöøùúûüýýþÿŔŕ?';
       $b = 'aaaaaaaceeeeiiiidnoooooouuuuybsaaaaaaaceeeeiiiidnoooooouuuuyybyRr-';
       $nome = strtr($nome, utf8_decode($a), $b);
       $nome = str_replace(".","-",$nome);
       $nome = preg_replace( "/[^0-9a-zA-Z\.]+/",'-',$nome);
       return utf8_decode(strtolower($nome.".".$extensao));
    }
    
    public function uploadXLS(Request\DataSet $request, Response $response){
        
        try {
            
            $rows = $request->getDataSet()->getRows()[0];
            
            $errors = '';
            $array = array(
                "path"     => '',
                "file"     => '',
                "status"   => '',
                'message'  => '',
                'client'   => '',
                'nameFile' => '',
                'content'  => ''
            );
            
            //extensoes permitidas
            $allowed_ext = array('xlsx', 'xls', 'ods', 'ots');
            
            $dirFile = "/tmp/";
            $success = '';
            
            if(!empty($rows['ARQEXCEL'])){
                foreach ($rows['ARQEXCEL'] as $key => $value) {
                    $array['nameFile'] = $value['name'];
                    $ext_type = explode('.', $this->tratarNomeArquivo(utf8_decode($value['name'])));
                    
                    if(in_array($ext_type[1], $allowed_ext) === false){
                         $errors .= ' Extensão não Permitida!';
                    }
                    
                    if($value['size'] > 1048576){
                        $errors .= ' Tamanho do arquivo deve ser até 1mb!';
                    }
                    
                    if(empty($errors)){
                        $base64 = explode(",", $value['b64File']);
                    	$data = base64_decode($base64[1]);
                    	$fileName = uniqid().'.'.$ext_type[1];
                    	$file = $dirFile.$fileName;
                    	$array['path'] = $dirFile;
                    	$array['file'] = $fileName;
                    	$array['client'] = $rows['CDCLIENTE'];
                    	$array['content'] = $base64[1];
                    	
                    	if(file_put_contents($file, $data)){
                    	    $array['status'] = true;
                    	    $array['message'] = 'Salvo com sucesso';
                    	}
                    }else{
                        $array['status'] = false;
                        $array['message'] = 'Não foi possível realizar o upload. '.$errors;
                    }
                }
            }
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('resultado', $array));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
        
    }
    
    public function deleteXLS(Request $request, Response $response){
        $rows = $request->getRow();
        $array = array(
            "status" => '',
            "message" => ''
        );
        if(!empty($rows['diretorio']) && !empty($rows['file'])){
            $path = $rows['diretorio'].$rows['file'];
            if(unlink($path)){
                $array['status'] = true;
                $array['message'] = "Deletado com sucesso!";
            }else{
                $array['status'] = false;
                $array['message'] = "Erro ao deletar com sucesso!";
            }
        }else{
            $array['status'] = false;
            $array['message'] = "Não há dados para conseguir deletar arquivo!";
        }
        
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('resultado', $array));
    }
    
    
    public function uploadArqfun(Request\DataSet $request, Response $response){
        try {
            
            $arrayReturn = array(
                "log" => array(),
                "rowsInsert" => 0,
                "rowsUpdate" => 0,
                "rowsError" => 0,
                "file" => '',
                "status" => true,
                'message' => '',
                'client' => '',
                'nameFile' => ''
            );
            
            $arrayReturn['log'][] = "#### Importação de Consumidores ####";
            $arrayReturn['log'][] = "";
            $arrayReturn['log'][] = "";
            $arrayReturn['log'][] = "";
            $arrayReturn['log'][] = "---------------------------------------------";
             
            $rows = $request->getDataSet()->getRows()[0];
            $base64 = explode(",", $rows['ARQTXT'][0]['b64File']);
            $data = base64_decode($base64[1]);
            $dataArray = explode(PHP_EOL, $data);
            $rows[':CDCLIENTE'] = $rows['CDCLIENTE'];
            
            $rows = $this->consumidorService->buscaCustoCliente($rows);

            if(!isset($rows[':CDCCUSCLIE'])){
                throw new \Exception('Cliente sem Centro de Custo cadastrado');
            }

            $rows = $this->consumidorService->buscaTipoConsumidorPadrao($rows);

            if(!isset($rows[':CDTIPOCONS'])){
                throw new \Exception('Tipo de Consumidor Padrão não Cadastrado');
            }
            
            $paramsToSave = self::clearParamsToSave($rows);
            $consumidores =  $this->consumidorService->getAllConsumidores($rows);
            $this->consumidorService->setIDSITCONSUMI();
            foreach($dataArray as $key => $rowData){
                    $paramsToSave = self::clearParamsToSave($rows);
                    if(strlen($rowData) == '94' && $rows['TPLAYOUT'] == 'P' && !empty($rowData)){
                        $paramsToSave[":IDENTIFICADOR"] = substr($rowData, 0, 1);
                        $paramsToSave[":CDEXCONSUMID"] = substr($rowData, 0, 9);
                        $paramsToSave[":CDIDCONSUMID"] = intval(substr($rowData, 9, 8));
                        $paramsToSave[":NMCONSUMIDOR"] = substr($rowData, 17, 30);
                        $paramsToSave[":CDEXCLIENTE"] = substr($rowData,65, 4 );
                        $paramsToSave[":VRLIMDEBCONS"] = null;
                        $paramsToSave[":VRAVIDEBCONS"] = null;
                        $paramsToSave[":IDVERSALDCON"] = 'S';
                        if(!empty($rows['VRLIMITCONSMENSAL'])){
                            $paramsToSave[":VRLIMDEBCONS"] = $this->valorBD($rows['VRLIMITCONSMENSAL']);
                            $paramsToSave[":VRMAXCONSMENSAL"] = null;
                        }else{
                            $paramsToSave[":VRMAXCONSMENSAL"] = $this->valorBD((substr($rowData, 69, 15 ))); 
                        }
                        

                        if($rows['IDIGNOREDAILYVALUE']=="S"){

                            if($rows['VRDAPPLYDAILYLIMITFOREVERYONE']!=""){
                                $paramsToSave[":VRMAXDEBCONS"] = $this->valorBD($rows['VRDAPPLYDAILYLIMITFOREVERYONE']);
                                //$rows[':VRMAXDEBCONS']=$rows['VRDAPPLYDAILYLIMITFOREVERYONE'];
                            }else{
                                $paramsToSave[":VRMAXDEBCONS"] = "";
                                //$rows[':VRMAXDEBCONS']="";
                            }

                        }else{
                            $paramsToSave[":VRMAXDEBCONS"] = $this->valorBD((substr($rowData, 84,9 )));
                        }

                        $this->consumidorService->saveConsumidorArqFun($paramsToSave, $consumidores, $arrayReturn);
                        
                    }else if((strlen($rowData) == '96' || strlen($rowData) == '97') && $rows['TPLAYOUT'] == 'N' && !empty($rowData)){
                        $paramsToSave[":IDENTIFICADOR"] = substr($rowData, 0, 1);
                        $paramsToSave[":CDEXCONSUMID"] = substr($rowData, 1, 9);
                        $paramsToSave[":CDIDCONSUMID"] = intval(substr($rowData, 10, 8));
                        $paramsToSave[":NMCONSUMIDOR"] = substr($rowData, 18, 30);
                        $paramsToSave[":CDEXCLIENTE"] = substr($rowData,67, 4 );
                        $paramsToSave[":VRLIMDEBCONS"] = null;
                        $paramsToSave[":VRAVIDEBCONS"] = null;
                        $paramsToSave[":IDVERSALDCON"] = 'S';
                        if(!empty($rows['VRLIMITCONSMENSAL'])){
                            $paramsToSave[":VRLIMDEBCONS"] = $this->valorBD($rows['VRLIMITCONSMENSAL']);
                            $paramsToSave[":VRMAXCONSMENSAL"] = null;
                        }else{
                            $paramsToSave[":VRMAXCONSMENSAL"] = $this->valorBD((substr($rowData, 72, 15 ))); 
                        }

                        if($rows['IDIGNOREDAILYVALUE']=="S"){
                            if($rows['VRDAPPLYDAILYLIMITFOREVERYONE']!=""){
                                $paramsToSave[":VRMAXDEBCONS"] = $this->valorBD($rows['VRDAPPLYDAILYLIMITFOREVERYONE']);
                                //$rows[':VRMAXDEBCONS']=$rows['VRDAPPLYDAILYLIMITFOREVERYONE'];
                            }else{
                                $paramsToSave[":VRMAXDEBCONS"] = "";
                                //$rows[':VRMAXDEBCONS']="";
                            }
                        }else{
                            $paramsToSave[":VRMAXDEBCONS"] = $this->valorBD((substr($rowData, 89,7)));
                            
                        }
                        
                        $this->consumidorService->saveConsumidorArqFun($paramsToSave, $consumidores, $arrayReturn);
                    }else if(!empty($rowData)){
                         throw new \Exception('Formato do arquivo inválido');
                    }
            }
            
            $arrayReturn['log'][1] = "Registros Inseridos: {$arrayReturn['rowsInsert']}";
            $arrayReturn['log'][2] = "Registros Atualizados: {$arrayReturn['rowsUpdate']}";
            $arrayReturn['log'][3] = "Registros não salvos: {$arrayReturn['rowsError']}";
            $arrayReturn = mb_convert_encoding($arrayReturn, 'UTF-8', 'UTF-8');
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('resultado', $arrayReturn));
            
        }catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        } 
    }
    
    public function invalidFormat($key, &$arrayReturn, $rowData){
        $arrayReturn['log'][6] .= PHP_EOL."Formato invalido na linha: ".($key+1); 
        if($rowData == ""){
            $arrayReturn['log'][6] .= PHP_EOL."Motivo: Linha vazia"; 
        } else {
            $arrayReturn['log'][6] .= PHP_EOL."Registro: ".$rowData; 
        }
        $arrayReturn['log'][] = "------ Formato invalido na linha: ".($key+1)." ------";
        $arrayReturn['status'] = false;
        $arrayReturn['rowsError']++;
    }
    
    public function clearParamsToSave($rows){
        return array(
                            ":CDCLIENTE"        =>$rows['CDCLIENTE'],
                            ":CDCCUSCLIE"       =>$rows[':CDCCUSCLIE'],
                            ":CDTIPOCONS"       =>$rows[':CDTIPOCONS'],
                            ":NRORG"            =>$rows['NRORG'],
                            ":IDENTIFICADOR"    =>'',
                            ":CDEXCONSUMID"     =>'',
                            ":CDIDCONSUMID"     =>'',
                            ":NMCONSUMIDOR"     =>'',
                            ":CDEXCLIENTE"      => '',
                            ":VRMAXCONSMENSAL"  => '',
                            ":VRMAXDEBCONS"     => ''
                        );
    }
    
    public function getParamsAsArray($params){
        $result = array();
        $result[':P_TCDFILIAL'] = 'T';
        $result[':P_CDFILIAL'] = '';
        $result[':P_TCDCLIENTE'] = 'T';
        $result[':P_CDCLIENTE'] = '';
        $result[':P_TCDTIPOCONS'] = 'T';
        $result[':P_CDTIPOCONS'] = '';
        $result[':P_TCDCONSUMIDOR'] = 'T';
        $result[':P_CDCONSUMIDOR'] = '';
        $result[':P_TCDPRODUTO'] = 'T';
        $result[':P_CDARVPRODI'] = '';
        $result[':P_CDARVPRODF'] = '';
        $result[':P_CDPRODLIST'] = '';
        $result[':P_ALLFIEDLS'] = 'T';
        
        if(!empty($params)){
            foreach($params as $key => $value){
                
                if(isset($value['name'])){
                    $value['columnName'] = $value['name'];
                }
                
                if($value['columnName'] == 'DTPERIODO'){
                    $date = explode('/', $value['value']);
                    $diaPrimeiroUltMes = '01'.'/'.(intval($date[0])).'/'.$date[1];
                    $ultimoDiaUltMes = date('t', strtotime(str_replace("/", "-", $diaPrimeiroUltMes))).'/'.(intval($date[0])).'/'.$date[1];
                    $result[':P_DTLANCINI'] = $diaPrimeiroUltMes;
                    $result[':P_DTLANCFIN'] = $ultimoDiaUltMes;
                }
                
                if($value['columnName'] == 'CDFILIAL'){
                    if(is_array($value['value'])){
                        $result[':P_TCDFILIAL'] = '';
                        $result[':P_CDFILIAL'] = implode('_', $value['value']);
                    }else{
                        $result[':P_TCDFILIAL'] = 'T';
                    }
                }
                
                if($value['columnName'] == 'CDCLIENTE'){
                    if(is_array($value['value'])){
                        $result[':P_TCDCLIENTE'] = '';
                        $result[':P_CDCLIENTE'] = implode('_', $value['value']);
                    }else{
                        $result[':P_TCDCLIENTE'] = 'T';
                    }
                }
                
                 if($value['columnName'] == 'CDTIPOCONS'){
                    if(is_array($value['value'])){
                        $result[':P_TCDTIPOCONS'] = '';
                        $result[':P_CDTIPOCONS'] = implode('_', $value['value']);
                    }else{
                        $result[':P_TCDTIPOCONS'] = 'T';
                    }
                }
                
                 if($value['columnName'] == 'CDCONSUMIDOR'){
                    if(is_array($value['value'])){
                        $result[':P_TCDCONSUMIDOR'] = '';
                        $result[':P_CDCONSUMIDOR'] = implode('_', $value['value']);
                    }else{
                        $result[':P_TCDCONSUMIDOR'] = 'T';
                    }
                }
                
                if($value['columnName'] == 'CDPRODUTO'){
                    $result[':P_CDPRODLIST'] = implode('_', $value['value']);
                    $result[':P_TCDPRODUTO'] = null;
                    $result[':P_CDARVPRODI'] = null;
                    $result[':P_CDARVPRODF'] = null;
                }
                
                if($value['columnName'] == 'CDARVPRODI'){
                    $result[':P_CDPRODLIST'] = null;
                    $result[':P_TCDPRODUTO'] = null;
                    $result[':P_CDARVPRODI'] = $value['value'] ;
                }
                
                if($value['columnName'] == 'CDARVPRODF'){
                    $result[':P_CDPRODLIST'] = null;
                    $result[':P_TCDPRODUTO'] = null;
                    $result[':P_CDARVPRODF'] = $value['value'];
                }
                
                if(strpos($value['columnName'], '|')){
                    $colunas = explode('|', $value['columnName']);
                    foreach($colunas as $k => $con){
                        $result[':P_ALLFIEDLS'] = $value['value'][0]['_ALL'];
                    }
                }
            }
        }
        return $result;
    }
    
    
    public function findVendasPorConsumidor(Request\Filter $request, Response $response) {
        $params = $request->getFilterCriteria()->getConditions();
        $params = $this->getParamsAsArray($params);
        $buscaVendasPorConsumidor = $this->consumidorService->buscaVendasPorConsumidor($params);
        $response->addDataSet(new DataSet("orm00033_relVendasPorConsumidor", $buscaVendasPorConsumidor));
        return $response;
    }
    
    public function exportarArq(Request $request, Response $response){
        $filters = $request->getRow();
        $params = $this->getParamsAsArray($filters['filter']);
        $text = '';
        $buscaVendasPorConsumidor = $this->consumidorService->buscaVendasPorConsumidor($params);
        
        if(!empty($buscaVendasPorConsumidor)){
            foreach($buscaVendasPorConsumidor as $key => $value){
                
                if($filters['type'] == 1){
                    $tmExCliente = 4;
                    $tmConsumo = 10;
                }else if($filters['type'] == 2){
                     $tmExCliente = 3;
                     $tmConsumo = 15;
                }
                
                if(empty($value['CDEXCLIENTE'])){
                    $text .= str_pad(substr($value['CDCLIENTE'], -$tmExCliente), $tmExCliente, "0", STR_PAD_LEFT);
                }else{
                   $text .= str_pad(substr($value['CDEXCLIENTE'], -$tmExCliente), $tmExCliente, "0", STR_PAD_LEFT);
                }
                
                if(empty($value['CDEXCONSUMID'])){
                    $text .= str_pad(substr($value['CDIDCONSUMID'], -9), 9, "0", STR_PAD_LEFT);
                }else{
                   $text .= str_pad(substr($value['CDEXCONSUMID'], -9), 9, "0", STR_PAD_LEFT);
                }
                
                if($filters['type'] == 1){
                   $text .= $value['DTPEOPLE'];
                }else if($filters['type'] == 2){
                   $text .= $value['DTNATCORP'];
                }
                
                $text .= '0679';
                $text .= str_pad(number_format($value['CONSUMO'], 2, '.', ''), $tmConsumo, "0", STR_PAD_LEFT);
                $text .= PHP_EOL;
            }
        }
        
        $return['texto'] = $text;
        
        if($filters['type'] == 1){
            $nomeArq = 'ExportPeople';
        }else if($filters['type'] == 2){
            $nomeArq = 'ExportNatCorp';
        }
        
        $return['nome'] = $nomeArq;
        $return['error'] = false;
        
        
        $response->addDataSet(new DataSet('resultado', $return));
    }
    
    private function valorBD($valorAplicado){
		
		$posVirgula = strpos( $valorAplicado, "," );
		$posPont = strpos( $valorAplicado, "." );
		
		if (!$posVirgula === false && !$posPont === false) {
			$valor = str_replace("." , "" , $valorAplicado ); // Primeiro tira os pontos
			$valorAplicado2 = str_replace(',','.',$valor);
		}else if(!$posVirgula === false && $posPont === false){
			$valorAplicado2 = str_replace(',','.',$valorAplicado);
		}else {
			$valorAplicado2 = $valorAplicado;
		}
		
		return $valorAplicado2;
	}
	
	public function requestRelatorio(DTO\Request\Row $request, DTO\Response $response) {
	    
	    $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
        $conditions = $request->getDataSet()->getRows()[0];
        //  $conditions = $this->util->getParamsAsArray($request);
        
        $params     = $this->consumidorService->buildReportParams($conditions);
       
        $this->getReport($params, $response);
    }
    
    
    
    public function getReport($param, DTO\Response $response){
            try {
                
                
                $params['ParametersRow'] = array( 'P_CDFILIAL'      =>$param['ParametersRow']['P_CDFILIAL'],
                                                  'P_TCDFILIAL'     =>$param['ParametersRow']['P_TCDFILIAL'],
                                                  'P_CDCLIENTE'     =>$param['ParametersRow']['P_CDCLIENTE'],
                                                  'P_TCDCLIENTE'    =>$param['ParametersRow']['P_TCDCLIENTE'],
                                                  'P_CDTIPOCONS'    =>$param['ParametersRow']['P_CDTIPOCONS'],
                                                  'P_TCDTIPOCONS'   =>$param['ParametersRow']['P_TCDTIPOCONS'],
                                                  'P_CDCONSUMIDOR'  =>$param['ParametersRow']['P_CDCONSUMIDOR'],
                                                  'P_TCDCONSUMIDOR' =>$param['ParametersRow']['P_TCDCONSUMIDOR'],
                                                  'P_NRORG'         =>$param['ParametersRow']['P_NRORG']
                                                  );
                                                  
                $params['ReportName']   = $param['ReportName'];
                
                
                $birtURL = $this->reportService->createRemoteReport('BIRT', $params['ReportName'], $params['ParametersRow']);
                
                if ($this->formatBirt === 'xls') {
                    $birtURL = str_replace('pdf', 'xls', $birtURL);
                }
                $response->addDataset(new DataSet('URL',array('url'=>$birtURL)));
            } catch (\Exception $e) {
                $response->setError(new Error($e->getMessage() , 500));
            }
        }
        
        
        
    public function anonimizarDadosConsumid(DTO\Request\Row $request, DTO\Response $response){
        try{
            $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
            $dataSet = $request->getDataSet()->getRows()[0];
            
            $params = array('CDCONSUMIDOR' => $dataSet['CDCONSUMIDOR'],
                            'CDCLIENTE'    => $dataSet['CDCLIENTE'],
                            'NRORG'        => $dataSet['NRORG']);
                            
            $resultCons = $this->consumidorService->getConsumidorAnon($this->consumidorService::GET_CONSTOANON,$params);
            
            if(!empty($resultCons)){
                
               $resultConsAnon = $this->consumidorService->getConsumidorAnon($this->consumidorService::GET_ANONIMIZACONS,$params); 
               
              
               
               if(empty($resultConsAnon)){
                    $resultCons = $resultCons[0];
                    $anonData = array(  'NRORG'           => $resultCons['NRORG'],
                                        'IDSITCONSUMI'    => $resultCons['IDSITCONSUMI'],
                                        'CDCONSUMIDOR'    => $resultCons['CDCONSUMIDOR'],
                                        'CDCLIENTE'       => $resultCons['CDCLIENTE'],
                                        'NMCONSUMIDOR'    => $this->consAnonimizator($resultCons['NMCONSUMIDOR'],'NAME'),
                                        'NMRESPCONS'      => $this->consAnonimizator($resultCons['NMRESPCONS'],'NAME'),
                                        'CDIDCONSUMID'    => $this->consAnonimizator($resultCons['CDIDCONSUMID'],'CRACHA'),
                                        'NRRGCONSUMID'    => $this->consAnonimizator($resultCons['NRRGCONSUMID'],'RG'),
                                        'NRCPFRESPCON'    => $this->consAnonimizator($resultCons['NRCPFRESPCON'],'CPF'),
                                        'DSEMAILCONS'     => $this->consAnonimizator($resultCons['DSEMAILCONS'],'EMAIL'),
                                        'NRTELECONS'      => $this->consAnonimizator($resultCons['NRTELECONS'],'TEL'),
                                        'NRTELE2CONS'     => $this->consAnonimizator($resultCons['NRTELE2CONS'],'TEL'),
                                        'NRCELULARCONS'   => $this->consAnonimizator($resultCons['NRCELULARCONS'],'TEL'),
                                        'DTNASCCONS'      => $resultCons['DTNASCCONS'],
                                        'DSCOMPLENDECONS' => $resultCons['DSCOMPLENDECONS'],
                                        'DSREFERENDECONS' => $resultCons['DSREFERENDECONS'],
                                        'DSENDECONS'      => $resultCons['DSENDECONS'],
                                        'CDBAIRRO'        => $resultCons['CDBAIRRO'],        
                                        'NRCEPCONS'       => $resultCons['NRCEPCONS']
                                        );
                  
                                        
                   $this->consumidorService->updateConsumidorAnon($anonData);
                        
                    $consAnonParams = array(    'CDCLIENTE'       =>    $resultCons['CDCLIENTE'],
                                                'CDCONSUMIDOR'    =>    $resultCons['CDCONSUMIDOR'],
                                                'CDOPERADOR'      =>    $dataSet['CDOPERADOR'],
                                                'NMOPERADOR'      =>    $dataSet['NMOPERADOR'],
                                                'NMARQANONIMIZA'  =>    $dataSet['ARQUIVO']['name'],
                                                'DSARQANONIMIZA'  =>    strval($dataSet['ARQUIVO']['b64File'])
                                            );
                                            
                    $this->consumidorService->insertConsumidorAnon($consAnonParams);
                    
                    $response->addDataset(new DataSet('anonimizarDados',array('status'=>'sucesso','CDCONSUMIDOR'=>$dataSet['CDCONSUMIDOR'])));
                           
                   
               }else{
                   $response->addDataset(new DataSet('anonimizarDados',array('status'=>'erro','message'=>' já anonimizado!','CDCONSUMIDOR'=>$dataSet['CDCONSUMIDOR'])));
               }
                
            }else{
                $response->addDataset(new DataSet('anonimizarDados',array('status'=>'erro','message'=>'ERRO! - Consumidor não encontrado','CDCONSUMIDOR'=>$dataSet['CDCONSUMIDOR'])));
            }
            
            
           
            
            
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage() , 500));
            
        }  
    }
    
    public function consAnonimizator ($str, $type){
        
        $strChars = str_split($str);
        $anonStr = '';
        foreach ($strChars as $i=>$char) {
            switch ($type) {
                case 'CRACHA':
                     $anonStr = $anonStr.'X';
                    break;
                case 'CPF':
                   ($i < 3) || ( $i+1 == count($strChars)) || ($i+1 == count($strChars) - 1 ) ? $anonStr = $anonStr.$char : $anonStr = $anonStr.'X';
                    break;
                case 'RG':
                    ($i < 2) || ($i+1 == count($strChars)) ? $anonStr = $anonStr.$char : $anonStr = $anonStr.'X';
                    break;
                case 'EMAIL':
                    ($i < 2) || ($char === '@') ? $anonStr = $anonStr.$char : $anonStr = $anonStr.'X';
                    break;
                case 'TEL':
                    ($char === '(') || ($char === ')') || ($char === '-') || ($i < 2) || ($i == 2 && $strChars[$i+1] === ')') || ( $i+1 == count($strChars)) || ($i+1 == count($strChars) - 1 ) ? $anonStr = $anonStr.$char : $anonStr = $anonStr.'X';
                    break;
                case 'NAME':
                    $initials = '';
                    $words = explode(' ', trim(preg_replace('/\s+/',' ',$str)));
                    foreach($words as $word){
                        if(!empty($word)){
                            $initials = strtoupper($initials.$word[0].' ');
                        }
                    }
                    
                    return trim($initials);
                    
            }
        }
        
        return $anonStr;
        
    }
    
    public function checkAnon(DTO\Request\Row $request, DTO\Response $response){
            try {
                $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
                $dataSet = $request->getDataSet()->getRows()[0];
                
                $params = array('CDCONSUMIDOR' => $dataSet['CDCONSUMIDOR'],
                                'CDCLIENTE'    => $dataSet['CDCLIENTE'],
                                'NRORG'        => $dataSet['NRORG']);
                                
                $resultCons = $this->consumidorService->getConsumidorAnon($this->consumidorService::GET_ANONIMIZACONS,$params);
                
                // print_r($resultCons);die;
                $response->addDataset(new DataSet('checkAnon',$resultCons));
                
                
            } catch (\Exception $e) {
                $response->setError(new Error($e->getMessage() , 500));
            }
        }
    
    public function findTemplateImportacao(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceTemplate;
        $this->parameterBagColumns = array('NRORG');
        parent::find($request, $response);
    }
    
}