<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;

class ParaVendaOnline extends \Zeedhi\Framework\Controller\Simple {

	protected $salesMessage;
    protected $paraVendaOnlineService;

	public function __construct(Services\ParaVendaOnline $paraVendaOnlineService, Utilities\SalesMessage $salesMessage, Utilities\Util $util) {
		$this->paraVendaOnlineService = $paraVendaOnlineService;
		$this->salesMessage = $salesMessage;
        $this->util = $util;        
	}

    public function parametrizarVendaGeral(Request\Filter $request, Response $response) {
        try {
            $params = $this->paraVendaOnlineService->parametrizarVendaGeral();

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('GetparaVendasGerais', $params));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function atualizaVendaGeral(Request\Filter $request, Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $IDCADPRODCMP      = $params[0]['value'];
            $DSPREVCONSPDV     = $params[1]['value'];
            $DSDIRSALVCAT      = $params[2]['value'];
            $DSDIRDESTARQTC    = $params[3]['value'];
            $IDUTTABPRECLVDC   = $params[4]['value'];
            $IDBLOQSEMIMP      = $params[5]['value'];
            $IDEXCONSATGER     = $params[6]['value'];
            $DSDIRARQXMLNFCE   = $params[7]['value'];
            
            $banco = $this->paraVendaOnlineService->atualizaVendaGeral($IDCADPRODCMP, $DSPREVCONSPDV, $DSDIRSALVCAT, $DSDIRDESTARQTC, $IDUTTABPRECLVDC, $IDBLOQSEMIMP, $IDEXCONSATGER, $DSDIRARQXMLNFCE);
            
            $response->addMessage(new Message($this->salesMessage->getMessage("atualizado")));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('nothing', array(array('nothing' => 'nothing'))));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function atualizaParaTEFMobile(Request\Filter $request, Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $IDTPTEF   = $params[0]['value'];
            $CDFILIAL = $params[1]['value'];
            $CDCAIXA = $params[2]['value'];
            $banco = $this->paraVendaOnlineService->atualizaParaTEFMobile($IDTPTEF, $CDFILIAL, $CDCAIXA);
            
            $response->addMessage(new Message($this->salesMessage->getMessage("atualizado")));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('nothing', array(array('nothing' => 'nothing'))));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function buscaAgencias(Request\Filter $request, Response $response) {
        try {
            $params  = $request->getFilterCriteria()->getConditions();
            $CDBANCO = $params[0]['value'];
            
            $agencias = $this->bancoService->buscaAgencias($CDBANCO);
            $mascara = $this->bancoService->buscaMascara($CDBANCO);
            
            if (empty($agencias)){
                // Não há agências cadastradas neste banco.
                $response->addMessage(new Message($this->salesMessage->getMessage(32)));
            } else {
                 if (empty($mascara)) {
                    // máscara não encontrada
                    $response->addMessage(new Message($this->salesMessage->getMessage(38)));
                } else {
                    $agencias = $this->mascara($agencias, $mascara[0]['CDPICTAGEN'], 'NMAGENCIA');
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BuscaAgenciasRepo', $agencias));
                }
            }
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function buscaContas(Request\Filter $request, Response $response) {
        try {
            $params    = $request->getFilterCriteria()->getConditions();
            $CDBANCO   = $params[0]['value'];
            $CDAGENCIA = $params[1]['value'];

            $contas = $this->bancoService->buscaContas($CDBANCO, $CDAGENCIA);
            $mascara = $this->bancoService->buscaMascara($CDBANCO);

            if (empty($contas)) {
                //Não há contas cadastradas nesta agência.
                $response->addMessage(new Message($this->salesMessage->getMessage(33)));
            } else {
                if (empty($mascara)) {
                    // máscara não encontrada
                    $response->addMessage(new Message($this->salesMessage->getMessage(38)));
                } else {
                    $contas = $this->mascara($contas, $mascara[0]['CDPICTCONT'], 'CDCONTCORR');
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BuscaContasRepo', $contas));
                }
            }
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function mascara($array, $mascara, $campo){
        $ndigitos = strpos($mascara, '-');
        if (($ndigitos) == false){
            return $array;
        } else {
            $idx = 0;
            while ($idx < sizeof($array)) {
                $contaAtual = $array[$idx][$campo];
                $array[$idx][$campo] = substr($contaAtual, 0, $ndigitos) . '-' . substr($contaAtual, ($ndigitos));
                $idx++;
            }
        }
        return $array;
    }

    public function getVendaFiliais(Request\Filter $request, Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $filiais = json_decode($params[0]['value']);
            
            // Override filiais with name and code
            foreach ($filiais as $key => $filial) {
                $filiais[$key] = $this->paraVendaOnlineService->getVendaFiliais($filial)[0];
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('GetVendaFiliaisRepository', $filiais));

        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
}